# generated by datamodel-codegen:
#   filename:  oscal_poam_schema.json
#   timestamp: 2023-03-16T10:34:00+00:00

from __future__ import annotations

from datetime import datetime
from enum import Enum
from typing import Annotated, List, Optional

from pydantic import AnyUrl, EmailStr, Extra, Field
from pydantic import BaseModel as PydanticBaseModel


class BaseModel(PydanticBaseModel):
    class Config:
        allow_population_by_field_name = True


class RelatedObservation(BaseModel):
    class Config:
        extra = Extra.forbid

    observation_uuid: Annotated[
        str,
        Field(
            alias="observation-uuid",
            description="A machine-oriented identifier reference to an observation defined in the list of observations.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Observation Universally Unique Identifier Reference",
        ),
    ]


class AssociatedRisk(BaseModel):
    class Config:
        extra = Extra.forbid

    risk_uuid: Annotated[
        str,
        Field(
            alias="risk-uuid",
            description="A machine-oriented identifier reference to a risk defined in the list of risks.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Risk Universally Unique Identifier Reference",
        ),
    ]


class LocationURL(BaseModel):
    __root__: Annotated[
        AnyUrl,
        Field(
            description="The uniform resource locator (URL) for a web site or Internet presence associated with the location.",
            # regex="^[a-zA-Z][a-zA-Z0-9+\\-.]+:.+$",
            title="Location URL",
        ),
    ]


class LocationReference(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="A machine-oriented identifier reference to a location defined in the metadata section of this or another OSCAL instance. The UUID of the location in the source OSCAL instance is sufficient to reference the data item locally or globally (e.g., in an imported OSCAL instance).",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Location Reference",
        ),
    ]


class PartyType(Enum):
    person = "person"
    organization = "organization"


class PartyExternalIdentifier(BaseModel):
    class Config:
        extra = Extra.forbid

    scheme: Annotated[
        AnyUrl,
        Field(
            description="Indicates the type of external identifier.",
            # regex="^[a-zA-Z][a-zA-Z0-9+\\-.]+:.+$",
            title="External Identifier Schema",
        ),
    ]
    id: str


class OrganizationalAffiliation(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="A machine-oriented identifier reference to another party (person or organization) that this subject is associated with. The UUID of the party in the source OSCAL instance is sufficient to reference the data item locally or globally (e.g., in an imported OSCAL instance).",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Organizational Affiliation",
        ),
    ]


class PartyReference(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="A machine-oriented identifier reference to another party defined in metadata. The UUID of the party in the source OSCAL instance is sufficient to reference the data item locally or globally (e.g., in an imported OSCAL instance).",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Party Reference",
        ),
    ]


class RoleIdentifierReference(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="A human-oriented identifier reference to roles served by the user.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Role Identifier Reference",
        ),
    ]


class Base64(BaseModel):
    class Config:
        extra = Extra.forbid

    filename: Annotated[
        Optional[str],
        Field(
            description="Name of the file before it was encoded as Base64 to be embedded in a resource. This is the name that will be assigned to the file when the file is decoded.",
            title="File Name",
        ),
    ] = None
    media_type: Annotated[
        Optional[str],
        Field(
            alias="media-type",
            description="Specifies a media type as defined by the Internet Assigned Numbers Authority (IANA) Media Types Registry.",
            regex="^\\S(.*\\S)?$",
            title="Media Type",
        ),
    ] = None
    value: str


class Link(BaseModel):
    class Config:
        extra = Extra.forbid

    href: Annotated[
        str,
        Field(
            description="A resolvable URL reference to a resource.",
            title="Hypertext Reference",
        ),
    ]
    rel: Annotated[
        Optional[str],
        Field(
            description="Describes the type of relationship provided by the link. This can be an indicator of the link's purpose.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Relation",
        ),
    ] = None
    media_type: Annotated[
        Optional[str],
        Field(
            alias="media-type",
            description="Specifies a media type as defined by the Internet Assigned Numbers Authority (IANA) Media Types Registry.",
            regex="^\\S(.*\\S)?$",
            title="Media Type",
        ),
    ] = None
    text: Annotated[
        Optional[str],
        Field(
            description="A textual label to associate with the link, which may be used for presentation in a tool.",
            title="Link Text",
        ),
    ] = None


class Hash(BaseModel):
    class Config:
        extra = Extra.forbid

    algorithm: Annotated[
        str,
        Field(
            description="Method by which a hash is derived",
            regex="^\\S(.*\\S)?$",
            title="Hash algorithm",
        ),
    ]
    value: str


class Remarks(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="Additional commentary on the containing object.",
            title="Remarks",
        ),
    ]


class PublicationTimestamp(BaseModel):
    __root__: Annotated[
        datetime,
        Field(
            description="The date and time the document was published. The date-time value must be formatted according to RFC 3339 with full time and time zone included.",
            # regex="^((2000|2400|2800|(19|2[0-9](0[48]|[2468][048]|[13579][26])))-02-29)|(((19|2[0-9])[0-9]{2})-02-(0[1-9]|1[0-9]|2[0-8]))|(((19|2[0-9])[0-9]{2})-(0[13578]|10|12)-(0[1-9]|[12][0-9]|3[01]))|(((19|2[0-9])[0-9]{2})-(0[469]|11)-(0[1-9]|[12][0-9]|30))T(2[0-3]|[01][0-9]):([0-5][0-9]):([0-5][0-9])(\\.[0-9]+)?(Z|[+-][0-9]{2}:[0-9]{2})$",
            title="Publication Timestamp",
        ),
    ]


class LastModifiedTimestamp(BaseModel):
    __root__: Annotated[
        datetime,
        Field(
            description="The date and time the document was last modified. The date-time value must be formatted according to RFC 3339 with full time and time zone included.",
            # regex="^((2000|2400|2800|(19|2[0-9](0[48]|[2468][048]|[13579][26])))-02-29)|(((19|2[0-9])[0-9]{2})-02-(0[1-9]|1[0-9]|2[0-8]))|(((19|2[0-9])[0-9]{2})-(0[13578]|10|12)-(0[1-9]|[12][0-9]|3[01]))|(((19|2[0-9])[0-9]{2})-(0[469]|11)-(0[1-9]|[12][0-9]|30))T(2[0-3]|[01][0-9]):([0-5][0-9]):([0-5][0-9])(\\.[0-9]+)?(Z|[+-][0-9]{2}:[0-9]{2})$",
            title="Last Modified Timestamp",
        ),
    ]


class DocumentVersion(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="A string used to distinguish the current version of the document from other previous (and future) versions.",
            regex="^\\S(.*\\S)?$",
            title="Document Version",
        ),
    ]


class OSCALVersion(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="The OSCAL model version the document was authored against.",
            regex="^\\S(.*\\S)?$",
            title="OSCAL version",
        ),
    ]


class EmailAddress(BaseModel):
    __root__: Annotated[
        EmailStr,
        Field(
            description="An email address as defined by RFC 5322 Section 3.4.1.",
            # regex="^.+@.+$",
            title="Email Address",
        ),
    ]


class TelephoneNumber(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[
        Optional[str],
        Field(
            description="Indicates the type of phone number.",
            regex="^\\S(.*\\S)?$",
            title="type flag",
        ),
    ] = None
    number: str


class AddressLine(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="A single line of an address.",
            regex="^\\S(.*\\S)?$",
            title="Address line",
        ),
    ]


class DocumentIdentifier(BaseModel):
    class Config:
        extra = Extra.forbid

    scheme: Annotated[
        Optional[AnyUrl],
        Field(
            description="Qualifies the kind of document identifier using a URI. If the scheme is not provided the value of the element will be interpreted as a string of characters.",
            # regex="^[a-zA-Z][a-zA-Z0-9+\\-.]+:.+$",
            title="Document Identification Scheme",
        ),
    ] = None
    identifier: str


class State(Enum):
    under_development = "under-development"
    operational = "operational"
    disposition = "disposition"
    other = "other"


class Status(BaseModel):
    class Config:
        extra = Extra.forbid

    state: Annotated[State, Field(description="The operational status.", title="State")]
    remarks: Optional[Remarks] = None


class Transport(Enum):
    TCP = "TCP"
    UDP = "UDP"


class PortRange(BaseModel):
    class Config:
        extra = Extra.forbid

    start: Annotated[
        Optional[int],
        Field(
            description="Indicates the starting port number in a port range",
            ge=0,
            multiple_of=1.0,
            title="Start",
        ),
    ] = None
    end: Annotated[
        Optional[int],
        Field(
            description="Indicates the ending port number in a port range",
            ge=0,
            multiple_of=1.0,
            title="End",
        ),
    ] = None
    transport: Annotated[
        Optional[Transport],
        Field(description="Indicates the transport type.", title="Transport"),
    ] = None


class ImplementationStatus(BaseModel):
    class Config:
        extra = Extra.forbid

    state: Annotated[
        str,
        Field(
            description="Identifies the implementation status of the control or control objective.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Implementation State",
        ),
    ]
    remarks: Optional[Remarks] = None


class FunctionsPerformed(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="Describes a function performed for a given authorized privilege by this user class.",
            regex="^\\S(.*\\S)?$",
            title="Functions Performed",
        ),
    ]


class ParameterValue(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="A parameter value or set of values.",
            regex="^\\S(.*\\S)?$",
            title="Parameter Value",
        ),
    ]


class SetParameterValue(BaseModel):
    class Config:
        extra = Extra.forbid

    param_id: Annotated[
        str,
        Field(
            alias="param-id",
            description="A human-oriented reference to a parameter within a control, who's catalog has been imported into the current implementation context.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Parameter ID",
        ),
    ]
    values: Annotated[List[ParameterValue], Field(min_items=1)]
    remarks: Optional[Remarks] = None


class SystemIdentification(BaseModel):
    class Config:
        extra = Extra.forbid

    identifier_type: Annotated[
        Optional[AnyUrl],
        Field(
            alias="identifier-type",
            description="Identifies the identification system from which the provided identifier was assigned.",
            # regex="^[a-zA-Z][a-zA-Z0-9+\\-.]+:.+$",
            title="Identification System Type",
        ),
    ] = None
    id: str


class ConstraintTest(BaseModel):
    class Config:
        extra = Extra.forbid

    expression: Annotated[
        str,
        Field(
            description="A formal (executable) expression of a constraint",
            regex="^\\S(.*\\S)?$",
            title="Constraint test",
        ),
    ]
    remarks: Optional[Remarks] = None


class Constraint(BaseModel):
    class Config:
        extra = Extra.forbid

    description: Annotated[
        Optional[str],
        Field(
            description="A textual summary of the constraint to be applied.",
            title="Constraint Description",
        ),
    ] = None
    tests: Annotated[Optional[List[ConstraintTest]], Field(min_items=1)] = None


class Guideline(BaseModel):
    class Config:
        extra = Extra.forbid

    prose: Annotated[
        str,
        Field(
            description="Prose permits multiple paragraphs, lists, tables etc.",
            title="Guideline Text",
        ),
    ]


class ParameterCardinality(Enum):
    one = "one"
    one_or_more = "one-or-more"


class Selection(BaseModel):
    class Config:
        extra = Extra.forbid

    how_many: Annotated[
        Optional[ParameterCardinality],
        Field(
            alias="how-many",
            description="Describes the number of selections that must occur. Without this setting, only one value should be assumed to be permitted.",
            title="Parameter Cardinality",
        ),
    ] = None
    choice: Annotated[Optional[List[str]], Field(min_items=1)] = None


class IncludeAll(BaseModel):
    pass

    class Config:
        extra = Extra.forbid


class ImportSystemSecurityPlan(BaseModel):
    class Config:
        extra = Extra.forbid

    href: Annotated[
        str,
        Field(
            description="A resolvable URL reference to the system security plan for the system being assessed.",
            title="System Security Plan Reference",
        ),
    ]
    remarks: Optional[Remarks] = None


class OnDateCondition(BaseModel):
    class Config:
        extra = Extra.forbid

    date: Annotated[
        datetime,
        Field(
            description="The task must occur on the specified date.",
            # regex="^((2000|2400|2800|(19|2[0-9](0[48]|[2468][048]|[13579][26])))-02-29)|(((19|2[0-9])[0-9]{2})-02-(0[1-9]|1[0-9]|2[0-8]))|(((19|2[0-9])[0-9]{2})-(0[13578]|10|12)-(0[1-9]|[12][0-9]|3[01]))|(((19|2[0-9])[0-9]{2})-(0[469]|11)-(0[1-9]|[12][0-9]|30))T(2[0-3]|[01][0-9]):([0-5][0-9]):([0-5][0-9])(\\.[0-9]+)?(Z|[+-][0-9]{2}:[0-9]{2})$",
            title="On Date Condition",
        ),
    ]


class OnDateRangeCondition(BaseModel):
    class Config:
        extra = Extra.forbid

    start: Annotated[
        datetime,
        Field(
            description="The task must occur on or after the specified date.",
            # regex="^((2000|2400|2800|(19|2[0-9](0[48]|[2468][048]|[13579][26])))-02-29)|(((19|2[0-9])[0-9]{2})-02-(0[1-9]|1[0-9]|2[0-8]))|(((19|2[0-9])[0-9]{2})-(0[13578]|10|12)-(0[1-9]|[12][0-9]|3[01]))|(((19|2[0-9])[0-9]{2})-(0[469]|11)-(0[1-9]|[12][0-9]|30))T(2[0-3]|[01][0-9]):([0-5][0-9]):([0-5][0-9])(\\.[0-9]+)?(Z|[+-][0-9]{2}:[0-9]{2})$",
            title="Start Date Condition",
        ),
    ]
    end: Annotated[
        datetime,
        Field(
            description="The task must occur on or before the specified date.",
            # regex="^((2000|2400|2800|(19|2[0-9](0[48]|[2468][048]|[13579][26])))-02-29)|(((19|2[0-9])[0-9]{2})-02-(0[1-9]|1[0-9]|2[0-8]))|(((19|2[0-9])[0-9]{2})-(0[13578]|10|12)-(0[1-9]|[12][0-9]|3[01]))|(((19|2[0-9])[0-9]{2})-(0[469]|11)-(0[1-9]|[12][0-9]|30))T(2[0-3]|[01][0-9]):([0-5][0-9]):([0-5][0-9])(\\.[0-9]+)?(Z|[+-][0-9]{2}:[0-9]{2})$",
            title="End Date Condition",
        ),
    ]


class TimeUnit(Enum):
    seconds = "seconds"
    minutes = "minutes"
    hours = "hours"
    days = "days"
    months = "months"
    years = "years"


class FrequencyCondition(BaseModel):
    class Config:
        extra = Extra.forbid

    period: Annotated[
        int,
        Field(
            description="The task must occur after the specified period has elapsed.",
            ge=1,
            multiple_of=1.0,
            title="Period",
        ),
    ]
    unit: Annotated[
        TimeUnit,
        Field(description="The unit of time for the period.", title="Time Unit"),
    ]


class EventTiming(BaseModel):
    class Config:
        extra = Extra.forbid

    on_date: Annotated[
        Optional[OnDateCondition],
        Field(
            alias="on-date",
            description="The task is intended to occur on the specified date.",
            title="On Date Condition",
        ),
    ] = None
    within_date_range: Annotated[
        Optional[OnDateRangeCondition],
        Field(
            alias="within-date-range",
            description="The task is intended to occur within the specified date range.",
            title="On Date Range Condition",
        ),
    ] = None
    at_frequency: Annotated[
        Optional[FrequencyCondition],
        Field(
            alias="at-frequency",
            description="The task is intended to occur at the specified frequency.",
            title="Frequency Condition",
        ),
    ] = None


class TaskDependency(BaseModel):
    class Config:
        extra = Extra.forbid

    task_uuid: Annotated[
        str,
        Field(
            alias="task-uuid",
            description="A machine-oriented identifier reference to a unique task.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Task Universally Unique Identifier Reference",
        ),
    ]
    remarks: Optional[Remarks] = None


class IncludeSpecificStatements(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="Used to constrain the selection to only specificity identified statements.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Include Specific Statements",
        ),
    ]


class SelectControl(BaseModel):
    class Config:
        extra = Extra.forbid

    control_id: Annotated[
        str,
        Field(
            alias="control-id",
            description="A human-oriented identifier reference to a control with a corresponding id value. When referencing an externally defined control, the Control Identifier Reference must be used in the context of the external / imported OSCAL instance (e.g., uri-reference).",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Control Identifier Reference",
        ),
    ]
    statement_ids: Annotated[
        Optional[List[IncludeSpecificStatements]],
        Field(alias="statement-ids", min_items=1),
    ] = None


class SelectObjective(BaseModel):
    class Config:
        extra = Extra.forbid

    objective_id: Annotated[
        str,
        Field(
            alias="objective-id",
            description="Points to an assessment objective.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Objective ID",
        ),
    ]


class AssessmentSubjectSource(BaseModel):
    class Config:
        extra = Extra.forbid

    task_uuid: Annotated[
        str,
        Field(
            alias="task-uuid",
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference (in this or other OSCAL instances) an assessment activity to be performed as part of the event. The locally defined UUID of the task can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Task Universally Unique Identifier",
        ),
    ]


class FindingTargetType(Enum):
    statement_id = "statement-id"
    objective_id = "objective-id"


class ObjectiveStatusState(Enum):
    satisfied = "satisfied"
    not_satisfied = "not-satisfied"


class ObjectiveStatus1(BaseModel):
    class Config:
        extra = Extra.forbid

    state: Annotated[
        ObjectiveStatusState,
        Field(
            description="An indication as to whether the objective is satisfied or not.",
            title="Objective Status State",
        ),
    ]
    reason: Annotated[
        Optional[str],
        Field(
            description="The reason the objective was given it's status.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Objective Status Reason",
        ),
    ] = None
    remarks: Optional[Remarks] = None


class ObservationMethod(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="Identifies how the observation was made.",
            regex="^\\S(.*\\S)?$",
            title="Observation Method",
        ),
    ]


class ObservationType(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="Identifies the nature of the observation. More than one may be used to further qualify and enable filtering.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Observation Type",
        ),
    ]


class ActorType(Enum):
    tool = "tool"
    assessment_platform = "assessment-platform"
    party = "party"


class ThreatID(BaseModel):
    class Config:
        extra = Extra.forbid

    system: Annotated[
        AnyUrl,
        Field(
            description="Specifies the source of the threat information.",
            # regex="^[a-zA-Z][a-zA-Z0-9+\\-.]+:.+$",
            title="Threat Type Identification System",
        ),
    ]
    href: Annotated[
        Optional[str],
        Field(
            description="An optional location for the threat data, from which this ID originates.",
            title="Threat Information Resource Reference",
        ),
    ] = None
    id: str


class LoggedBy(BaseModel):
    class Config:
        extra = Extra.forbid

    party_uuid: Annotated[
        str,
        Field(
            alias="party-uuid",
            description="A machine-oriented identifier reference to the party who is making the log entry.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Party UUID Reference",
        ),
    ]
    role_id: Annotated[
        Optional[str],
        Field(
            alias="role-id",
            description="A point to the role-id of the role in which the party is making the log entry.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Actor Role",
        ),
    ] = None


class RiskStatus(BaseModel):
    __root__: Annotated[
        str,
        Field(
            description="Describes the status of the associated risk.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Risk Status",
        ),
    ]


class ResourceLink(BaseModel):
    class Config:
        extra = Extra.forbid

    href: Annotated[
        str,
        Field(
            description="A resolvable URI reference to a resource.",
            title="Hypertext Reference",
        ),
    ]
    media_type: Annotated[
        Optional[str],
        Field(
            alias="media-type",
            description="Specifies a media type as defined by the Internet Assigned Numbers Authority (IANA) Media Types Registry.",
            regex="^\\S(.*\\S)?$",
            title="Media Type",
        ),
    ] = None
    hashes: Annotated[Optional[List[Hash]], Field(min_items=1)] = None


class Property(BaseModel):
    class Config:
        extra = Extra.forbid

    name: Annotated[
        str,
        Field(
            description="A textual label that uniquely identifies a specific attribute, characteristic, or quality of the property's containing object.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Property Name",
        ),
    ]
    uuid: Annotated[
        Optional[str],
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this defined property elsewhere in this or other OSCAL instances. This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Property Universally Unique Identifier",
        ),
    ] = None
    ns: Annotated[
        Optional[AnyUrl],
        Field(
            description="A namespace qualifying the property's name. This allows different organizations to associate distinct semantics with the same name.",
            # regex="^[a-zA-Z][a-zA-Z0-9+\\-.]+:.+$",
            title="Property Namespace",
        ),
    ] = None
    value: Annotated[
        str,
        Field(
            description="Indicates the value of the attribute, characteristic, or quality.",
            regex="^\\S(.*\\S)?$",
            title="Property Value",
        ),
    ]
    class_: Annotated[
        Optional[str],
        Field(
            alias="class",
            description="A textual label that provides a sub-type or characterization of the property's name. This can be used to further distinguish or discriminate between the semantics of multiple properties of the same object with the same name and ns.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Property Class",
        ),
    ] = None
    remarks: Optional[Remarks] = None


class ResponsibleParty(BaseModel):
    class Config:
        extra = Extra.forbid

    role_id: Annotated[
        str,
        Field(
            alias="role-id",
            description="A human-oriented identifier reference to roles served by the user.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Responsible Role",
        ),
    ]
    party_uuids: Annotated[List[PartyReference], Field(alias="party-uuids", min_items=1)]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    remarks: Optional[Remarks] = None


class ResponsibleRole(BaseModel):
    class Config:
        extra = Extra.forbid

    role_id: Annotated[
        str,
        Field(
            alias="role-id",
            description="A human-oriented identifier reference to roles responsible for the business function.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Responsible Role ID",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    party_uuids: Annotated[
        Optional[List[PartyReference]], Field(alias="party-uuids", min_items=1)
    ] = None
    remarks: Optional[Remarks] = None


class Address(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[
        Optional[str],
        Field(
            description="Indicates the type of address.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Address Type",
        ),
    ] = None
    addr_lines: Annotated[
        Optional[List[AddressLine]], Field(alias="addr-lines", min_items=1)
    ] = None
    city: Annotated[
        Optional[str],
        Field(
            description="City, town or geographical region for the mailing address.",
            regex="^\\S(.*\\S)?$",
            title="City",
        ),
    ] = None
    state: Annotated[
        Optional[str],
        Field(
            description="State, province or analogous geographical region for mailing address",
            regex="^\\S(.*\\S)?$",
            title="State",
        ),
    ] = None
    postal_code: Annotated[
        Optional[str],
        Field(
            alias="postal-code",
            description="Postal or ZIP code for mailing address",
            regex="^\\S(.*\\S)?$",
            title="Postal Code",
        ),
    ] = None
    country: Annotated[
        Optional[str],
        Field(
            description="The ISO 3166-1 alpha-2 country code for the mailing address.",
            regex="^\\S(.*\\S)?$",
            title="Country Code",
        ),
    ] = None


class ServiceProtocolInformation(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        Optional[str],
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this service protocol information elsewhere in this or other OSCAL instances. The locally defined UUID of the service protocol can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Service Protocol Information Universally Unique Identifier",
        ),
    ] = None
    name: Annotated[
        str,
        Field(
            description='The common name of the protocol, which should be the appropriate "service name" from the IANA Service Name and Transport Protocol Port Number Registry.',
            regex="^\\S(.*\\S)?$",
            title="Protocol Name",
        ),
    ]
    title: Annotated[
        Optional[str],
        Field(
            description="A human readable name for the protocol (e.g., Transport Layer Security).",
            title="Protocol Title",
        ),
    ] = None
    port_ranges: Annotated[
        Optional[List[PortRange]], Field(alias="port-ranges", min_items=1)
    ] = None


class Privilege(BaseModel):
    class Config:
        extra = Extra.forbid

    title: Annotated[
        str,
        Field(
            description="A human readable name for the privilege.",
            title="Privilege Title",
        ),
    ]
    description: Annotated[
        Optional[str],
        Field(
            description="A summary of the privilege's purpose within the system.",
            title="Privilege Description",
        ),
    ] = None
    functions_performed: Annotated[
        List[FunctionsPerformed], Field(alias="functions-performed", min_items=1)
    ]


class ImplementedComponent(BaseModel):
    class Config:
        extra = Extra.forbid

    component_uuid: Annotated[
        str,
        Field(
            alias="component-uuid",
            description="A machine-oriented identifier reference to a component that is implemented as part of an inventory item.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Component Universally Unique Identifier Reference",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    responsible_parties: Annotated[
        Optional[List[ResponsibleParty]],
        Field(alias="responsible-parties", min_items=1),
    ] = None
    remarks: Optional[Remarks] = None


class InventoryItem(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this inventory item elsewhere in this or other OSCAL instances. The locally defined UUID of the inventory item can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Inventory Item Universally Unique Identifier",
        ),
    ]
    description: Annotated[
        str,
        Field(
            description="A summary of the inventory item stating its purpose within the system.",
            title="Inventory Item Description",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    responsible_parties: Annotated[
        Optional[List[ResponsibleParty]],
        Field(alias="responsible-parties", min_items=1),
    ] = None
    implemented_components: Annotated[
        Optional[List[ImplementedComponent]],
        Field(alias="implemented-components", min_items=1),
    ] = None
    remarks: Optional[Remarks] = None


class Part(BaseModel):
    class Config:
        extra = Extra.forbid

    id: Annotated[
        Optional[str],
        Field(
            description="A human-oriented, locally unique identifier with cross-instance scope that can be used to reference this defined part elsewhere in this or other OSCAL instances. When referenced from another OSCAL instance, this identifier must be referenced in the context of the containing resource (e.g., import-profile). This id should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Part Identifier",
        ),
    ] = None
    name: Annotated[
        str,
        Field(
            description="A textual label that uniquely identifies the part's semantic type.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Part Name",
        ),
    ]
    ns: Annotated[
        Optional[AnyUrl],
        Field(
            description="A namespace qualifying the part's name. This allows different organizations to associate distinct semantics with the same name.",
            # regex="^[a-zA-Z][a-zA-Z0-9+\\-.]+:.+$",
            title="Part Namespace",
        ),
    ] = None
    class_: Annotated[
        Optional[str],
        Field(
            alias="class",
            description="A textual label that provides a sub-type or characterization of the part's name. This can be used to further distinguish or discriminate between the semantics of multiple parts of the same control with the same name and ns.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Part Class",
        ),
    ] = None
    title: Annotated[
        Optional[str],
        Field(
            description="A name given to the part, which may be used by a tool for display and navigation.",
            title="Part Title",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    prose: Annotated[
        Optional[str],
        Field(
            description="Permits multiple paragraphs, lists, tables etc.",
            title="Part Text",
        ),
    ] = None
    parts: Optional[List[Part]] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None


class Parameter(BaseModel):
    class Config:
        extra = Extra.forbid

    id: Annotated[
        str,
        Field(
            description="A human-oriented, locally unique identifier with cross-instance scope that can be used to reference this defined parameter elsewhere in this or other OSCAL instances. When referenced from another OSCAL instance, this identifier must be referenced in the context of the containing resource (e.g., import-profile). This id should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Parameter Identifier",
        ),
    ]
    class_: Annotated[
        Optional[str],
        Field(
            alias="class",
            description="A textual label that provides a characterization of the parameter.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Parameter Class",
        ),
    ] = None
    depends_on: Annotated[
        Optional[str],
        Field(
            alias="depends-on",
            description="**(deprecated)** Another parameter invoking this one. This construct has been deprecated and should not be used.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Depends on",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    label: Annotated[
        Optional[str],
        Field(
            description="A short, placeholder name for the parameter, which can be used as a substitute for a value if no value is assigned.",
            title="Parameter Label",
        ),
    ] = None
    usage: Annotated[
        Optional[str],
        Field(
            description="Describes the purpose and use of a parameter",
            title="Parameter Usage Description",
        ),
    ] = None
    constraints: Annotated[Optional[List[Constraint]], Field(min_items=1)] = None
    guidelines: Annotated[Optional[List[Guideline]], Field(min_items=1)] = None
    values: Annotated[Optional[List[ParameterValue]], Field(min_items=1)] = None
    select: Optional[Selection] = None
    remarks: Optional[Remarks] = None


class AssessmentSpecificControlObjective(BaseModel):
    class Config:
        extra = Extra.forbid

    control_id: Annotated[
        str,
        Field(
            alias="control-id",
            description="A human-oriented identifier reference to a control with a corresponding id value. When referencing an externally defined control, the Control Identifier Reference must be used in the context of the external / imported OSCAL instance (e.g., uri-reference).",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Control Identifier Reference",
        ),
    ]
    description: Annotated[
        Optional[str],
        Field(
            description="A human-readable description of this control objective.",
            title="Objective Description",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    parts: Annotated[List[Part], Field(min_items=1)]
    remarks: Optional[Remarks] = None


class AssessedControls(BaseModel):
    class Config:
        extra = Extra.forbid

    description: Annotated[
        Optional[str],
        Field(
            description="A human-readable description of in-scope controls specified for assessment.",
            title="Assessed Controls Description",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    include_all: Annotated[Optional[IncludeAll], Field(alias="include-all")] = None
    include_controls: Annotated[
        Optional[List[SelectControl]], Field(alias="include-controls", min_items=1)
    ] = None
    exclude_controls: Annotated[
        Optional[List[SelectControl]], Field(alias="exclude-controls", min_items=1)
    ] = None
    remarks: Optional[Remarks] = None


class ReferencedControlObjectives(BaseModel):
    class Config:
        extra = Extra.forbid

    description: Annotated[
        Optional[str],
        Field(
            description="A human-readable description of this collection of control objectives.",
            title="Control Objectives Description",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    include_all: Annotated[Optional[IncludeAll], Field(alias="include-all")] = None
    include_objectives: Annotated[
        Optional[List[SelectObjective]], Field(alias="include-objectives", min_items=1)
    ] = None
    exclude_objectives: Annotated[
        Optional[List[SelectObjective]], Field(alias="exclude-objectives", min_items=1)
    ] = None
    remarks: Optional[Remarks] = None


class ReviewedControlsAndControlObjectives(BaseModel):
    class Config:
        extra = Extra.forbid

    description: Annotated[
        Optional[str],
        Field(
            description="A human-readable description of control objectives.",
            title="Control Objective Description",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    control_selections: Annotated[
        List[AssessedControls], Field(alias="control-selections", min_items=1)
    ]
    control_objective_selections: Annotated[
        Optional[List[ReferencedControlObjectives]],
        Field(alias="control-objective-selections", min_items=1),
    ] = None
    remarks: Optional[Remarks] = None


class AssessmentSubjectPlaceholder(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier for a set of assessment subjects that will be identified by a task or an activity that is part of a task. The locally defined UUID of the assessment subject placeholder can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Assessment Subject Placeholder Universally Unique Identifier",
        ),
    ]
    description: Annotated[
        Optional[str],
        Field(
            description="A human-readable description of intent of this assessment subject placeholder.",
            title="Assessment Subject Placeholder Description",
        ),
    ] = None
    sources: Annotated[List[AssessmentSubjectSource], Field(min_items=1)]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    remarks: Optional[Remarks] = None


class SelectAssessmentSubject(BaseModel):
    class Config:
        extra = Extra.forbid

    subject_uuid: Annotated[
        str,
        Field(
            alias="subject-uuid",
            description="A machine-oriented identifier reference to a component, inventory-item, location, party, user, or resource using it's UUID.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Subject Universally Unique Identifier Reference",
        ),
    ]
    type: Annotated[
        str,
        Field(
            description="Used to indicate the type of object pointed to by the uuid-ref within a subject.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Subject Universally Unique Identifier Reference Type",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    remarks: Optional[Remarks] = None


class IdentifiesTheSubject(BaseModel):
    class Config:
        extra = Extra.forbid

    subject_uuid: Annotated[
        str,
        Field(
            alias="subject-uuid",
            description="A machine-oriented identifier reference to a component, inventory-item, location, party, user, or resource using it's UUID.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Subject Universally Unique Identifier Reference",
        ),
    ]
    type: Annotated[
        str,
        Field(
            description="Used to indicate the type of object pointed to by the uuid-ref within a subject.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Subject Universally Unique Identifier Reference Type",
        ),
    ]
    title: Annotated[
        Optional[str],
        Field(
            description="The title or name for the referenced subject.",
            title="Subject Reference Title",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    remarks: Optional[Remarks] = None


class UsesComponent(BaseModel):
    class Config:
        extra = Extra.forbid

    component_uuid: Annotated[
        str,
        Field(
            alias="component-uuid",
            description="A machine-oriented identifier reference to a component that is implemented as part of an inventory item.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Component Universally Unique Identifier Reference",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    responsible_parties: Annotated[
        Optional[List[ResponsibleParty]],
        Field(alias="responsible-parties", min_items=1),
    ] = None
    remarks: Optional[Remarks] = None


class AssessmentPlatform(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this assessment platform elsewhere in this or other OSCAL instances. The locally defined UUID of the assessment platform can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Assessment Platform Universally Unique Identifier",
        ),
    ]
    title: Annotated[
        Optional[str],
        Field(
            description="The title or name for the assessment platform.",
            title="Assessment Platform Title",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    uses_components: Annotated[
        Optional[List[UsesComponent]], Field(alias="uses-components", min_items=1)
    ] = None
    remarks: Optional[Remarks] = None


class ObjectiveStatus(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[
        FindingTargetType,
        Field(
            description="Identifies the type of the target.",
            title="Finding Target Type",
        ),
    ]
    target_id: Annotated[
        str,
        Field(
            alias="target-id",
            description="A machine-oriented identifier reference for a specific target qualified by the type.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Finding Target Identifier Reference",
        ),
    ]
    title: Annotated[
        Optional[str],
        Field(
            description="The title for this objective status.",
            title="Objective Status Title",
        ),
    ] = None
    description: Annotated[
        Optional[str],
        Field(
            description="A human-readable description of the assessor's conclusions regarding the degree to which an objective is satisfied.",
            title="Objective Status Description",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    status: Annotated[
        ObjectiveStatus1,
        Field(
            description="A determination of if the objective is satisfied or not within a given system.",
            title="Objective Status",
        ),
    ]
    implementation_status: Annotated[
        Optional[ImplementationStatus], Field(alias="implementation-status")
    ] = None
    remarks: Optional[Remarks] = None


class RelevantEvidence(BaseModel):
    class Config:
        extra = Extra.forbid

    href: Annotated[
        Optional[str],
        Field(
            description="A resolvable URL reference to relevant evidence.",
            title="Relevant Evidence Reference",
        ),
    ] = None
    description: Annotated[
        str,
        Field(
            description="A human-readable description of this evidence.",
            title="Relevant Evidence Description",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    remarks: Optional[Remarks] = None


class OriginatingActor(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[ActorType, Field(description="The kind of actor.", title="Actor Type")]
    actor_uuid: Annotated[
        str,
        Field(
            alias="actor-uuid",
            description="A machine-oriented identifier reference to the tool or person based on the associated type.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Actor Universally Unique Identifier Reference",
        ),
    ]
    role_id: Annotated[
        Optional[str],
        Field(
            alias="role-id",
            description="For a party, this can optionally be used to specify the role the actor was performing.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Actor Role",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None


class MitigatingFactor(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this mitigating factor elsewhere in this or other OSCAL instances. The locally defined UUID of the mitigating factor can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Mitigating Factor Universally Unique Identifier",
        ),
    ]
    implementation_uuid: Annotated[
        Optional[str],
        Field(
            alias="implementation-uuid",
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this implementation statement elsewhere in this or other OSCAL instancess. The locally defined UUID of the implementation statement can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Implementation UUID",
        ),
    ] = None
    description: Annotated[
        str,
        Field(
            description="A human-readable description of this mitigating factor.",
            title="Mitigating Factor Description",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    subjects: Annotated[Optional[List[IdentifiesTheSubject]], Field(min_items=1)] = None


class Facet(BaseModel):
    class Config:
        extra = Extra.forbid

    name: Annotated[
        str,
        Field(
            description="The name of the risk metric within the specified system.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Facet Name",
        ),
    ]
    system: Annotated[
        AnyUrl,
        Field(
            description="Specifies the naming system under which this risk metric is organized, which allows for the same names to be used in different systems controlled by different parties. This avoids the potential of a name clash.",
            # regex="^[a-zA-Z][a-zA-Z0-9+\\-.]+:.+$",
            title="Naming System",
        ),
    ]
    value: Annotated[
        str,
        Field(
            description="Indicates the value of the facet.",
            regex="^\\S(.*\\S)?$",
            title="Facet Value",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    remarks: Optional[Remarks] = None


class RequiredAsset(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this required asset elsewhere in this or other OSCAL instances. The locally defined UUID of the asset can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Required Universally Unique Identifier",
        ),
    ]
    subjects: Annotated[Optional[List[IdentifiesTheSubject]], Field(min_items=1)] = None
    title: Annotated[
        Optional[str],
        Field(
            description="The title for this required asset.",
            title="Title for Required Asset",
        ),
    ] = None
    description: Annotated[
        str,
        Field(
            description="A human-readable description of this required asset.",
            title="Description of Required Asset",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    remarks: Optional[Remarks] = None


class AssessmentPart(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        Optional[str],
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this part elsewhere in this or other OSCAL instances. The locally defined UUID of the part can be used to reference the data item locally or globally (e.g., in an ported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Part Identifier",
        ),
    ] = None
    name: Annotated[
        str,
        Field(
            description="A textual label that uniquely identifies the part's semantic type.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Part Name",
        ),
    ]
    ns: Annotated[
        Optional[AnyUrl],
        Field(
            description="A namespace qualifying the part's name. This allows different organizations to associate distinct semantics with the same name.",
            # regex="^[a-zA-Z][a-zA-Z0-9+\\-.]+:.+$",
            title="Part Namespace",
        ),
    ] = None
    class_: Annotated[
        Optional[str],
        Field(
            alias="class",
            description="A textual label that provides a sub-type or characterization of the part's name. This can be used to further distinguish or discriminate between the semantics of multiple parts of the same control with the same name and ns.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Part Class",
        ),
    ] = None
    title: Annotated[
        Optional[str],
        Field(
            description="A name given to the part, which may be used by a tool for display and navigation.",
            title="Part Title",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    prose: Annotated[
        Optional[str],
        Field(
            description="Permits multiple paragraphs, lists, tables etc.",
            title="Part Text",
        ),
    ] = None
    parts: Optional[List[AssessmentPart]] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None


class Origin(BaseModel):
    class Config:
        extra = Extra.forbid

    actors: Annotated[List[OriginatingActor], Field(min_items=1)]


class POAMItem(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        Optional[str],
        Field(
            description="A machine-oriented, globally unique identifier with instance scope that can be used to reference this POA&M item entry in this OSCAL instance. This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="POA&M Item Universally Unique Identifier",
        ),
    ] = None
    title: Annotated[
        str,
        Field(
            description="The title or name for this POA&M item .",
            title="POA&M Item Title",
        ),
    ]
    description: Annotated[
        str,
        Field(
            description="A human-readable description of POA&M item.",
            title="POA&M Item Description",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    origins: Annotated[Optional[List[Origin]], Field(min_items=1)] = None
    related_observations: Annotated[
        Optional[List[RelatedObservation]],
        Field(alias="related-observations", min_items=1),
    ] = None
    related_risks: Annotated[
        Optional[List[AssociatedRisk]], Field(alias="related-risks", min_items=1)
    ] = None
    remarks: Optional[Remarks] = None


class RevisionHistoryEntry(BaseModel):
    class Config:
        extra = Extra.forbid

    title: Annotated[
        Optional[str],
        Field(
            description="A name given to the document revision, which may be used by a tool for display and navigation.",
            title="Document Title",
        ),
    ] = None
    published: Optional[PublicationTimestamp] = None
    last_modified: Annotated[Optional[LastModifiedTimestamp], Field(alias="last-modified")] = None
    version: DocumentVersion
    oscal_version: Annotated[Optional[OSCALVersion], Field(alias="oscal-version")] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    remarks: Optional[Remarks] = None


class Location(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this defined location elsewhere in this or other OSCAL instances. The locally defined UUID of the location can be used to reference the data item locally or globally (e.g., from an importing OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Location Universally Unique Identifier",
        ),
    ]
    title: Annotated[
        Optional[str],
        Field(
            description="A name given to the location, which may be used by a tool for display and navigation.",
            title="Location Title",
        ),
    ] = None
    address: Address
    email_addresses: Annotated[
        Optional[List[EmailAddress]], Field(alias="email-addresses", min_items=1)
    ] = None
    telephone_numbers: Annotated[
        Optional[List[TelephoneNumber]], Field(alias="telephone-numbers", min_items=1)
    ] = None
    urls: Annotated[Optional[List[LocationURL]], Field(min_items=1)] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    remarks: Optional[Remarks] = None


class PartyOrganizationOrPerson(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this defined party elsewhere in this or other OSCAL instances. The locally defined UUID of the party can be used to reference the data item locally or globally (e.g., from an importing OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Party Universally Unique Identifier",
        ),
    ]
    type: Annotated[
        PartyType,
        Field(
            description="A category describing the kind of party the object describes.",
            title="Party Type",
        ),
    ]
    name: Annotated[
        Optional[str],
        Field(
            description="The full name of the party. This is typically the legal name associated with the party.",
            regex="^\\S(.*\\S)?$",
            title="Party Name",
        ),
    ] = None
    short_name: Annotated[
        Optional[str],
        Field(
            alias="short-name",
            description="A short common name, abbreviation, or acronym for the party.",
            regex="^\\S(.*\\S)?$",
            title="Party Short Name",
        ),
    ] = None
    external_ids: Annotated[
        Optional[List[PartyExternalIdentifier]],
        Field(alias="external-ids", min_items=1),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    email_addresses: Annotated[
        Optional[List[EmailAddress]], Field(alias="email-addresses", min_items=1)
    ] = None
    telephone_numbers: Annotated[
        Optional[List[TelephoneNumber]], Field(alias="telephone-numbers", min_items=1)
    ] = None
    addresses: Annotated[Optional[List[Address]], Field(min_items=1)] = None
    location_uuids: Annotated[
        Optional[List[LocationReference]], Field(alias="location-uuids", min_items=1)
    ] = None
    member_of_organizations: Annotated[
        Optional[List[OrganizationalAffiliation]],
        Field(alias="member-of-organizations", min_items=1),
    ] = None
    remarks: Optional[Remarks] = None


class Role(BaseModel):
    class Config:
        extra = Extra.forbid

    id: Annotated[
        str,
        Field(
            description="A human-oriented, locally unique identifier with cross-instance scope that can be used to reference this defined role elsewhere in this or other OSCAL instances. When referenced from another OSCAL instance, the locally defined ID of the Role from the imported OSCAL instance must be referenced in the context of the containing resource (e.g., import, import-component-definition, import-profile, import-ssp or import-ap). This ID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Role Identifier",
        ),
    ]
    title: Annotated[
        str,
        Field(
            description="A name given to the role, which may be used by a tool for display and navigation.",
            title="Role Title",
        ),
    ]
    short_name: Annotated[
        Optional[str],
        Field(
            alias="short-name",
            description="A short common name, abbreviation, or acronym for the role.",
            regex="^\\S(.*\\S)?$",
            title="Role Short Name",
        ),
    ] = None
    description: Annotated[
        Optional[str],
        Field(
            description="A summary of the role's purpose and associated responsibilities.",
            title="Role Description",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    remarks: Optional[Remarks] = None


class Citation(BaseModel):
    class Config:
        extra = Extra.forbid

    text: Annotated[str, Field(description="A line of citation text.", title="Citation Text")]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None


class Resource(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this defined resource elsewhere in this or other OSCAL instances. This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Resource Universally Unique Identifier",
        ),
    ]
    title: Annotated[
        Optional[str],
        Field(
            description="A name given to the resource, which may be used by a tool for display and navigation.",
            title="Resource Title",
        ),
    ] = None
    description: Annotated[
        Optional[str],
        Field(
            description="A short summary of the resource used to indicate the purpose of the resource.",
            title="Resource Description",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    document_ids: Annotated[
        Optional[List[DocumentIdentifier]], Field(alias="document-ids", min_items=1)
    ] = None
    citation: Annotated[
        Optional[Citation],
        Field(
            description="A citation consisting of end note text and optional structured bibliographic data.",
            title="Citation",
        ),
    ] = None
    rlinks: Annotated[Optional[List[ResourceLink]], Field(min_items=1)] = None
    base64: Annotated[
        Optional[Base64],
        Field(
            description="The Base64 alphabet in RFC 2045 - aligned with XSD.",
            title="Base64",
        ),
    ] = None
    remarks: Optional[Remarks] = None


class BackMatter(BaseModel):
    class Config:
        extra = Extra.forbid

    resources: Annotated[Optional[List[Resource]], Field(min_items=1)] = None


class Component(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this component elsewhere in this or other OSCAL instances. The locally defined UUID of the component can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Component Identifier",
        ),
    ]
    type: Annotated[
        str,
        Field(
            description="A category describing the purpose of the component.",
            regex="^\\S(.*\\S)?$",
            title="Component Type",
        ),
    ]
    title: Annotated[
        str,
        Field(
            description="A human readable name for the system component.",
            title="Component Title",
        ),
    ]
    description: Annotated[
        str,
        Field(
            description="A description of the component, including information about its function.",
            title="Component Description",
        ),
    ]
    purpose: Annotated[
        Optional[str],
        Field(
            description="A summary of the technological or business purpose of the component.",
            title="Purpose",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    status: Annotated[
        Status,
        Field(
            description="Describes the operational status of the system component.",
            title="Status",
        ),
    ]
    responsible_roles: Annotated[
        Optional[List[ResponsibleRole]], Field(alias="responsible-roles", min_items=1)
    ] = None
    protocols: Annotated[Optional[List[ServiceProtocolInformation]], Field(min_items=1)] = None
    remarks: Optional[Remarks] = None


class SystemUser(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this user class elsewhere in this or other OSCAL instances. The locally defined UUID of the system user can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="User Universally Unique Identifier",
        ),
    ]
    title: Annotated[
        Optional[str],
        Field(
            description="A name given to the user, which may be used by a tool for display and navigation.",
            title="User Title",
        ),
    ] = None
    short_name: Annotated[
        Optional[str],
        Field(
            alias="short-name",
            description="A short common name, abbreviation, or acronym for the user.",
            regex="^\\S(.*\\S)?$",
            title="User Short Name",
        ),
    ] = None
    description: Annotated[
        Optional[str],
        Field(
            description="A summary of the user's purpose within the system.",
            title="User Description",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    role_ids: Annotated[
        Optional[List[RoleIdentifierReference]], Field(alias="role-ids", min_items=1)
    ] = None
    authorized_privileges: Annotated[
        Optional[List[Privilege]], Field(alias="authorized-privileges", min_items=1)
    ] = None
    remarks: Optional[Remarks] = None


class AssessmentMethod(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this assessment method elsewhere in this or other OSCAL instances. The locally defined UUID of the assessment method can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Assessment Method Universally Unique Identifier",
        ),
    ]
    description: Annotated[
        Optional[str],
        Field(
            description="A human-readable description of this assessment method.",
            title="Assessment Method Description",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    part: AssessmentPart
    remarks: Optional[Remarks] = None


class Step(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this step elsewhere in this or other OSCAL instances. The locally defined UUID of the step (in a series of steps) can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Step Universally Unique Identifier",
        ),
    ]
    title: Annotated[
        Optional[str], Field(description="The title for this step.", title="Step Title")
    ] = None
    description: Annotated[
        str,
        Field(
            description="A human-readable description of this step.",
            title="Step Description",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    reviewed_controls: Annotated[
        Optional[ReviewedControlsAndControlObjectives], Field(alias="reviewed-controls")
    ] = None
    responsible_roles: Annotated[
        Optional[List[ResponsibleRole]], Field(alias="responsible-roles", min_items=1)
    ] = None
    remarks: Optional[Remarks] = None


class Activity(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this assessment activity elsewhere in this or other OSCAL instances. The locally defined UUID of the activity can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Assessment Activity Universally Unique Identifier",
        ),
    ]
    title: Annotated[
        Optional[str],
        Field(
            description="The title for this included activity.",
            title="Included Activity Title",
        ),
    ] = None
    description: Annotated[
        str,
        Field(
            description="A human-readable description of this included activity.",
            title="Included Activity Description",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    steps: Annotated[Optional[List[Step]], Field(min_items=1)] = None
    related_controls: Annotated[
        Optional[ReviewedControlsAndControlObjectives], Field(alias="related-controls")
    ] = None
    responsible_roles: Annotated[
        Optional[List[ResponsibleRole]], Field(alias="responsible-roles", min_items=1)
    ] = None
    remarks: Optional[Remarks] = None


class SubjectOfAssessment(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Annotated[
        str,
        Field(
            description="Indicates the type of assessment subject, such as a component, inventory, item, location, or party represented by this selection statement.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Subject Type",
        ),
    ]
    description: Annotated[
        Optional[str],
        Field(
            description="A human-readable description of the collection of subjects being included in this assessment.",
            title="Include Subjects Description",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    include_all: Annotated[Optional[IncludeAll], Field(alias="include-all")] = None
    include_subjects: Annotated[
        Optional[List[SelectAssessmentSubject]],
        Field(alias="include-subjects", min_items=1),
    ] = None
    exclude_subjects: Annotated[
        Optional[List[SelectAssessmentSubject]],
        Field(alias="exclude-subjects", min_items=1),
    ] = None
    remarks: Optional[Remarks] = None


class AssessmentAssets(BaseModel):
    class Config:
        extra = Extra.forbid

    components: Annotated[Optional[List[Component]], Field(min_items=1)] = None
    assessment_platforms: Annotated[
        List[AssessmentPlatform], Field(alias="assessment-platforms", min_items=1)
    ]


class IdentifiedSubject(BaseModel):
    class Config:
        extra = Extra.forbid

    subject_placeholder_uuid: Annotated[
        str,
        Field(
            alias="subject-placeholder-uuid",
            description="A machine-oriented identifier reference to a unique assessment subject placeholder defined by this task.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Assessment Subject Placeholder Universally Unique Identifier Reference",
        ),
    ]
    subjects: Annotated[List[SubjectOfAssessment], Field(min_items=1)]


class TaskReference(BaseModel):
    class Config:
        extra = Extra.forbid

    task_uuid: Annotated[
        str,
        Field(
            alias="task-uuid",
            description="A machine-oriented identifier reference to a unique task.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Task Universally Unique Identifier Reference",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    responsible_parties: Annotated[
        Optional[List[ResponsibleParty]],
        Field(alias="responsible-parties", min_items=1),
    ] = None
    subjects: Annotated[Optional[List[SubjectOfAssessment]], Field(min_items=1)] = None
    identified_subject: Annotated[
        Optional[IdentifiedSubject],
        Field(
            alias="identified-subject",
            description="Used to detail assessment subjects that were identfied by this task.",
            title="Identified Subject",
        ),
    ] = None
    remarks: Optional[Remarks] = None


class RiskResponseReference(BaseModel):
    class Config:
        extra = Extra.forbid

    response_uuid: Annotated[
        str,
        Field(
            alias="response-uuid",
            description="A machine-oriented identifier reference to a unique risk response.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Response Universally Unique Identifier Reference",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    related_tasks: Annotated[
        Optional[List[TaskReference]], Field(alias="related-tasks", min_items=1)
    ] = None
    remarks: Optional[Remarks] = None


class RiskLogEntry(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this risk log entry elsewhere in this or other OSCAL instances. The locally defined UUID of the risk log entry can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Risk Log Entry Universally Unique Identifier",
        ),
    ]
    title: Annotated[
        Optional[str],
        Field(description="The title for this risk log entry.", title="Title"),
    ] = None
    description: Annotated[
        Optional[str],
        Field(
            description="A human-readable description of what was done regarding the risk.",
            title="Risk Task Description",
        ),
    ] = None
    start: Annotated[
        datetime,
        Field(
            description="Identifies the start date and time of the event.",
            # regex="^((2000|2400|2800|(19|2[0-9](0[48]|[2468][048]|[13579][26])))-02-29)|(((19|2[0-9])[0-9]{2})-02-(0[1-9]|1[0-9]|2[0-8]))|(((19|2[0-9])[0-9]{2})-(0[13578]|10|12)-(0[1-9]|[12][0-9]|3[01]))|(((19|2[0-9])[0-9]{2})-(0[469]|11)-(0[1-9]|[12][0-9]|30))T(2[0-3]|[01][0-9]):([0-5][0-9]):([0-5][0-9])(\\.[0-9]+)?(Z|[+-][0-9]{2}:[0-9]{2})$",
            title="Start",
        ),
    ]
    end: Annotated[
        Optional[datetime],
        Field(
            description="Identifies the end date and time of the event. If the event is a point in time, the start and end will be the same date and time.",
            # regex="^((2000|2400|2800|(19|2[0-9](0[48]|[2468][048]|[13579][26])))-02-29)|(((19|2[0-9])[0-9]{2})-02-(0[1-9]|1[0-9]|2[0-8]))|(((19|2[0-9])[0-9]{2})-(0[13578]|10|12)-(0[1-9]|[12][0-9]|3[01]))|(((19|2[0-9])[0-9]{2})-(0[469]|11)-(0[1-9]|[12][0-9]|30))T(2[0-3]|[01][0-9]):([0-5][0-9]):([0-5][0-9])(\\.[0-9]+)?(Z|[+-][0-9]{2}:[0-9]{2})$",
            title="End",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    logged_by: Annotated[Optional[List[LoggedBy]], Field(alias="logged-by", min_items=1)] = None
    status_change: Annotated[Optional[RiskStatus], Field(alias="status-change")] = None
    related_responses: Annotated[
        Optional[List[RiskResponseReference]],
        Field(alias="related-responses", min_items=1),
    ] = None
    remarks: Optional[Remarks] = None


class RiskLog(BaseModel):
    class Config:
        extra = Extra.forbid

    entries: Annotated[List[RiskLogEntry], Field(min_items=1)]


class LocalDefinitions(BaseModel):
    class Config:
        extra = Extra.forbid

    components: Annotated[Optional[List[Component]], Field(min_items=1)] = None
    inventory_items: Annotated[
        Optional[List[InventoryItem]], Field(alias="inventory-items", min_items=1)
    ] = None
    remarks: Optional[Remarks] = None


class PublicationMetadata(BaseModel):
    class Config:
        extra = Extra.forbid

    title: Annotated[
        str,
        Field(
            description="A name given to the document, which may be used by a tool for display and navigation.",
            title="Document Title",
        ),
    ]
    published: Optional[PublicationTimestamp] = None
    last_modified: Annotated[LastModifiedTimestamp, Field(alias="last-modified")]
    version: DocumentVersion
    oscal_version: Annotated[OSCALVersion, Field(alias="oscal-version")]
    revisions: Annotated[Optional[List[RevisionHistoryEntry]], Field(min_items=1)] = None
    document_ids: Annotated[
        Optional[List[DocumentIdentifier]], Field(alias="document-ids", min_items=1)
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    roles: Annotated[Optional[List[Role]], Field(min_items=1)] = None
    locations: Annotated[Optional[List[Location]], Field(min_items=1)] = None
    parties: Annotated[Optional[List[PartyOrganizationOrPerson]], Field(min_items=1)] = None
    responsible_parties: Annotated[
        Optional[List[ResponsibleParty]],
        Field(alias="responsible-parties", min_items=1),
    ] = None
    remarks: Optional[Remarks] = None


class AssociatedActivity(BaseModel):
    class Config:
        extra = Extra.forbid

    activity_uuid: Annotated[
        str,
        Field(
            alias="activity-uuid",
            description="A machine-oriented identifier reference to an activity defined in the list of activities.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Activity Universally Unique Identifier Reference",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    responsible_roles: Annotated[
        Optional[List[ResponsibleRole]], Field(alias="responsible-roles", min_items=1)
    ] = None
    subjects: Annotated[List[SubjectOfAssessment], Field(min_items=1)]
    remarks: Optional[Remarks] = None


class Task(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this task elsewhere in this or other OSCAL instances. The locally defined UUID of the task can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Task Universally Unique Identifier",
        ),
    ]
    type: Annotated[
        str,
        Field(
            description="The type of task.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Task Type",
        ),
    ]
    title: Annotated[str, Field(description="The title for this task.", title="Task Title")]
    description: Annotated[
        Optional[str],
        Field(
            description="A human-readable description of this task.",
            title="Task Description",
        ),
    ] = None
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    timing: Annotated[
        Optional[EventTiming],
        Field(
            description="The timing under which the task is intended to occur.",
            title="Event Timing",
        ),
    ] = None
    dependencies: Annotated[Optional[List[TaskDependency]], Field(min_items=1)] = None
    tasks: Optional[List[Task]] = None
    associated_activities: Annotated[
        Optional[List[AssociatedActivity]],
        Field(alias="associated-activities", min_items=1),
    ] = None
    subjects: Annotated[Optional[List[SubjectOfAssessment]], Field(min_items=1)] = None
    responsible_roles: Annotated[
        Optional[List[ResponsibleRole]], Field(alias="responsible-roles", min_items=1)
    ] = None
    remarks: Optional[Remarks] = None


class Origin1(BaseModel):
    class Config:
        extra = Extra.forbid

    actors: Annotated[List[OriginatingActor], Field(min_items=1)]
    related_tasks: Annotated[
        Optional[List[TaskReference]], Field(alias="related-tasks", min_items=1)
    ] = None


class Characterization(BaseModel):
    class Config:
        extra = Extra.forbid

    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    origin: Origin1
    facets: Annotated[List[Facet], Field(min_items=1)]


class RiskResponse(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this remediation elsewhere in this or other OSCAL instances. The locally defined UUID of the risk response can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Remediation Universally Unique Identifier",
        ),
    ]
    lifecycle: Annotated[
        str,
        Field(
            description="Identifies whether this is a recommendation, such as from an assessor or tool, or an actual plan accepted by the system owner.",
            regex="^(\\w|_)(\\w|\\d|[.\\--_])*$",
            title="Remediation Intent",
        ),
    ]
    title: Annotated[
        str,
        Field(description="The title for this response activity.", title="Response Title"),
    ]
    description: Annotated[
        str,
        Field(
            description="A human-readable description of this response plan.",
            title="Response Description",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    origins: Annotated[Optional[List[Origin1]], Field(min_items=1)] = None
    required_assets: Annotated[
        Optional[List[RequiredAsset]], Field(alias="required-assets", min_items=1)
    ] = None
    tasks: Annotated[Optional[List[Task]], Field(min_items=1)] = None
    remarks: Optional[Remarks] = None


class Observation(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this observation elsewhere in this or other OSCAL instances. The locally defined UUID of the observation can be used to reference the data item locally or globally (e.g., in an imorted OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Observation Universally Unique Identifier",
        ),
    ]
    title: Annotated[
        Optional[str],
        Field(description="The title for this observation.", title="Observation Title"),
    ] = None
    description: Annotated[
        str,
        Field(
            description="A human-readable description of this assessment observation.",
            title="Observation Description",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    methods: Annotated[List[ObservationMethod], Field(min_items=1)]
    types: Annotated[Optional[List[ObservationType]], Field(min_items=1)] = None
    origins: Annotated[Optional[List[Origin1]], Field(min_items=1)] = None
    subjects: Annotated[Optional[List[IdentifiesTheSubject]], Field(min_items=1)] = None
    relevant_evidence: Annotated[
        Optional[List[RelevantEvidence]], Field(alias="relevant-evidence", min_items=1)
    ] = None
    collected: Annotated[
        datetime,
        Field(
            description="Date/time stamp identifying when the finding information was collected.",
            # regex="^((2000|2400|2800|(19|2[0-9](0[48]|[2468][048]|[13579][26])))-02-29)|(((19|2[0-9])[0-9]{2})-02-(0[1-9]|1[0-9]|2[0-8]))|(((19|2[0-9])[0-9]{2})-(0[13578]|10|12)-(0[1-9]|[12][0-9]|3[01]))|(((19|2[0-9])[0-9]{2})-(0[469]|11)-(0[1-9]|[12][0-9]|30))T(2[0-3]|[01][0-9]):([0-5][0-9]):([0-5][0-9])(\\.[0-9]+)?(Z|[+-][0-9]{2}:[0-9]{2})$",
            title="Collected Field",
        ),
    ]
    expires: Annotated[
        Optional[datetime],
        Field(
            description="Date/time identifying when the finding information is out-of-date and no longer valid. Typically used with continuous assessment scenarios.",
            # regex="^((2000|2400|2800|(19|2[0-9](0[48]|[2468][048]|[13579][26])))-02-29)|(((19|2[0-9])[0-9]{2})-02-(0[1-9]|1[0-9]|2[0-8]))|(((19|2[0-9])[0-9]{2})-(0[13578]|10|12)-(0[1-9]|[12][0-9]|3[01]))|(((19|2[0-9])[0-9]{2})-(0[469]|11)-(0[1-9]|[12][0-9]|30))T(2[0-3]|[01][0-9]):([0-5][0-9]):([0-5][0-9])(\\.[0-9]+)?(Z|[+-][0-9]{2}:[0-9]{2})$",
            title="Expires Field",
        ),
    ] = None
    remarks: Optional[Remarks] = None


class IdentifiedRisk(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with cross-instance scope that can be used to reference this risk elsewhere in this or other OSCAL instances. The locally defined UUID of the risk can be used to reference the data item locally or globally (e.g., in an imported OSCAL instance). This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="Risk Universally Unique Identifier",
        ),
    ]
    title: Annotated[str, Field(description="The title for this risk.", title="Risk Title")]
    description: Annotated[
        str,
        Field(
            description="A human-readable summary of the identified risk, to include a statement of how the risk impacts the system.",
            title="Risk Description",
        ),
    ]
    statement: Annotated[
        str,
        Field(
            description="An summary of impact for how the risk affects the system.",
            title="Risk Statement",
        ),
    ]
    props: Annotated[Optional[List[Property]], Field(min_items=1)] = None
    links: Annotated[Optional[List[Link]], Field(min_items=1)] = None
    status: RiskStatus
    origins: Annotated[Optional[List[Origin1]], Field(min_items=1)] = None
    threat_ids: Annotated[Optional[List[ThreatID]], Field(alias="threat-ids", min_items=1)] = None
    characterizations: Annotated[Optional[List[Characterization]], Field(min_items=1)] = None
    mitigating_factors: Annotated[
        Optional[List[MitigatingFactor]], Field(alias="mitigating-factors", min_items=1)
    ] = None
    deadline: Annotated[
        Optional[datetime],
        Field(
            description="The date/time by which the risk must be resolved.",
            # regex="^((2000|2400|2800|(19|2[0-9](0[48]|[2468][048]|[13579][26])))-02-29)|(((19|2[0-9])[0-9]{2})-02-(0[1-9]|1[0-9]|2[0-8]))|(((19|2[0-9])[0-9]{2})-(0[13578]|10|12)-(0[1-9]|[12][0-9]|3[01]))|(((19|2[0-9])[0-9]{2})-(0[469]|11)-(0[1-9]|[12][0-9]|30))T(2[0-3]|[01][0-9]):([0-5][0-9]):([0-5][0-9])(\\.[0-9]+)?(Z|[+-][0-9]{2}:[0-9]{2})$",
            title="Risk Resolution Deadline",
        ),
    ] = None
    remediations: Annotated[Optional[List[RiskResponse]], Field(min_items=1)] = None
    risk_log: Annotated[
        Optional[RiskLog],
        Field(
            alias="risk-log",
            description="A log of all risk-related tasks taken.",
            title="Risk Log",
        ),
    ] = None
    related_observations: Annotated[
        Optional[List[RelatedObservation]],
        Field(alias="related-observations", min_items=1),
    ] = None


class PlanOfActionAndMilestonesPOAM(BaseModel):
    class Config:
        extra = Extra.forbid

    uuid: Annotated[
        str,
        Field(
            description="A machine-oriented, globally unique identifier with instancescope that can be used to reference this POA&M instance in this OSCAL instance. This UUID should be assigned per-subject, which means it should be consistently used to identify the same subject across revisions of the document.",
            regex="^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[45][0-9A-Fa-f]{3}-[89ABab][0-9A-Fa-f]{3}-[0-9A-Fa-f]{12}$",
            title="POA&M Universally Unique Identifier",
        ),
    ]
    metadata: PublicationMetadata
    import_ssp: Annotated[Optional[ImportSystemSecurityPlan], Field(alias="import-ssp")] = None
    system_id: Annotated[Optional[SystemIdentification], Field(alias="system-id")] = None
    local_definitions: Annotated[
        Optional[LocalDefinitions], Field(alias="local-definitions")
    ] = None
    observations: Annotated[Optional[List[Observation]], Field(min_items=1)] = None
    risks: Annotated[Optional[List[IdentifiedRisk]], Field(min_items=1)] = None
    poam_items: Annotated[List[POAMItem], Field(alias="poam-items", min_items=1)]
    back_matter: Annotated[Optional[BackMatter], Field(alias="back-matter")] = None


class Model(BaseModel):
    class Config:
        extra = Extra.forbid

    plan_of_action_and_milestones: Annotated[
        PlanOfActionAndMilestonesPOAM, Field(alias="plan-of-action-and-milestones")
    ]


Part.update_forward_refs()
AssessmentPart.update_forward_refs()
Task.update_forward_refs()
